// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

#include "dictionary.h"

#include "KVpair.h"

#include "salist.h"

// Dictionary implemented with a sorted array-based list
template <typename Key, typename Elem>
class SALdict : public Dictionary<Key, Elem> {
private:
  SAList<Key,Elem>* list;
public:
  SALdict(int size=DefaultListSize)   // Constructor
    { list = new SAList<Key,Elem>(size); }
  ~SALdict() { delete list; }         // Destructor
  void clear() { list->clear(); }     // Reinitialize

  // Insert an element: Keep elements sorted
  void insert(const Key&k, const Elem& e) {
    KVpair<Key,Elem> temp(k, e);
    list->insert(temp);
  }

  // Use sequential search to find the element to remove
  bool remove(const Key& K, Elem& e) {
    if(find(K, e)) { // Use "find" to set list position
      KVpair<Key,Elem> temp = list->remove();
      return true;
    }
    return false;
  }

  Elem removeAny() { // Remove the last element
    Assert(size() != 0, "Dictionary is empty");
    list->movetoEnd();
    list->prev();
    KVpair<Key,Elem> e = list->remove();
    return e.value();
  }

  // Find "K" using binary search
  bool find(const Key& K, Elem& e) const {
    int l = -1;
    int r = list->leftLength() + list->rightLength();
    while (l+1 != r) { // Stop when l and r meet
      int i = (l+r)/2; // Check middle of remaining subarray
      list->movetoPos(i);
      KVpair<Key,Elem> temp = list->getValue();
      if (K < temp.key()) r = i;        // In left
      if (K == temp.key()) return true; // Found it
      if (K > temp.key()) l = i;        // In right
    }
    return false; // "K" does not appear in dictionary
  }
  int size() // Return list size
    { return list->leftLength() + list->rightLength(); }
};
