// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// This is the file to include in your code if you want access to the
// complete AList template class

// First, get the declaration for the base list class
#include "list.h"

// This is the declaration for AList. It is split into two parts
// because it is too big to fit on one book page
template <typename Elem> // Array-based list implementation
class AList : public List<Elem> {
private:
  int maxSize;        // Maximum size of list
  int listSize;       // Actual number of elements in list
  int fence;          // Position of fence
  Elem* listArray;    // Array holding list elements

public:
  AList(int size=DefaultListSize) { // Constructor
    maxSize = size;
    listSize = fence = 0;
    listArray = new Elem[maxSize];
  }

  ~AList() { delete [] listArray; } // Destructor

  void clear() {                    // Reinitialize the list
    delete [] listArray;            // Remove the array
    listSize = fence = 0;           // Reset the size
    listArray = new Elem[maxSize];  // Recreate array
  }

  // Insert "it" at front of right partition
  bool insert(const Elem& it) {
    if (listSize < maxSize) {
      for(int i=listSize; i>fence; i--)    // Shift Elems up
        listArray[i] = listArray[i-1];     //   to make room
      listArray[fence] = it;
      listSize++;                     // Increment list size
      return true;
    }
    else // List is full
      return false;
  }

  bool append(const Elem& it) {       // Append "it"
    if (listSize < maxSize) {
      listArray[listSize++] = it;
      return true;
    }
    else // List is full
      return false;
  }

  // Remove and return first Elem in right partition
  Elem remove() {
    Assert(rightLength() > 0, "Nothing to remove");
    Elem it = listArray[fence];           // Copy the Elem
    for(int i=fence; i<listSize-1; i++)   // Shift them down
      listArray[i] = listArray[i+1];
    listSize--;                           // Decrement size
    return it;
  }

  void movetoStart() { fence = 0; }          // Reset fence
  void movetoEnd() { fence = listSize; }     // Reset fence
  void prev() { if (fence != 0) fence--; }       // Back up
  void next() { if (fence < listSize) fence++; } // Next

  // Return left or right partition size
  int leftLength() const  { return fence; }
  int rightLength() const { return listSize - fence; }

  void movetoPos(int pos) {  // Reset list position to "pos"
    Assert ((pos>=0)&&(pos<=listSize), "Pos out of range");
    fence = pos;
  }

  const Elem& getValue() const { // Return current Elem
    Assert(rightLength() > 0, "Nothing to get");
    return listArray[fence];
  }
};
