// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// The Dictionary abstract class.
template <typename Key, typename Elem>
class  Dictionary {
private:
  void operator =(const Dictionary&) {}
  Dictionary(const Dictionary&) {}

public:
  Dictionary() {}          // Default constructor
  virtual ~Dictionary() {} // Base destructor

  // Reinitialize dictionary
  virtual void clear() = 0;

  // Insert "e" into the dictionary.
  virtual void insert(const Key& k, const Elem& e) = 0;

  // Remove some record with key "k" and return it in "e".
  // Return true if such exists, false otherwise.
  // If multiple records match "k", remove an arbitrary one.
  virtual bool remove(const Key& k, Elem& e) = 0;

  // Remove and return an arbitrary record from dictionary.
  virtual Elem removeAny() = 0;

  // Return a copy in "e" of some record matching "k".
  // Return true if such exists, false otherwise. If
  // multiple records match "k", return an arbitrary one.
  virtual bool find(const Key& k, Elem& e) const = 0;

  // Return the number of records in the dictionary.
  virtual int size() = 0;
};
