// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// This is the file to include in your code if you want access to the
// complete DList template class

// First, get the declaration for the base list class
#include "list.h"

// This is the declaration for DList. It is broken up because the
// methods that appear in the book are in a separate file.
// Linked list implementation
template <typename Elem> class LList: public List<Elem> {
private:
  Link<Elem>* head;       // Pointer to list header
  Link<Elem>* tail;       // Pointer to list tailer
  Link<Elem>* fence;      // Last element on left side
  int leftcnt;            // Size of left partition
  int rightcnt;           // Size of right partition

  void init() {           // Intialization helper method
    fence = head = new Link<Elem>;
    tail = new Link<Elem>(head, NULL);
    leftcnt = rightcnt = 0;
  }

  void removeall() {   // Return link nodes to free store
    while(head != NULL) {
      fence = head;
      head = head->next;
      delete fence;
    }
  }

public:
  LList(int size=DefaultListSize) { init(); } // Constructor
  ~LList() { removeall(); }  // Destructor    // Destructor
  void clear() { removeall(); init(); }       // Clear list

  void movetoStart() // Place fence at list start
    { fence = head; rightcnt += leftcnt; leftcnt = 0; }

  void movetoEnd()   // Place fence at list end
    { fence = tail->prev; leftcnt += rightcnt; rightcnt = 0; }

  // Move fence one step right; no change if right is empty
  void next() {
    if (fence != tail->prev)
      { fence = fence->next; rightcnt--; leftcnt++; }
  }

  int leftLength() const  { return leftcnt; }
  int rightLength() const { return rightcnt; }

  // Set the size of left partition to "pos"
  void movetoPos(int pos) {
    Assert ((pos>=0)&&(pos<=rightcnt+leftcnt), "Position out of range");
    rightcnt = rightcnt + leftcnt - pos; // Set counts
    leftcnt = pos;
    fence = head;
    for(int i=0; i<pos; i++) fence = fence->next;
  }

  const Elem& getValue() const { // Return current Elem
    Assert(rightLength() > 0, "Nothing to remove");
    return fence->next->element;
  }

// Include those  methods that are different from singly linked list
// Insert "it" at front of the right partition
bool insert(const Elem& it) {
  new Link<Elem>(it, fence, fence->next);  
  rightcnt++;                    // Added to right
  return true;
}

// Append "it" to the end of the list.
bool append(const Elem& it) {
  new Link<Elem>(it, tail->prev, tail);
  rightcnt++;                    // Added to right
  return true;
}

// Remove and return first Elem in right partition
Elem remove() {
  Assert(rightLength() > 0, "Nothing to remove");
  Elem it = fence->next->element;  // Remember value
  Link<Elem>* ltemp = fence->next; // Remember link node
  ltemp->next->prev = fence;
  fence->next = ltemp->next;       // Remove from list
  delete ltemp;                    // Reclaim space
  rightcnt--;                      // Removed from right
  return it;
}

// Move fence one step left; no change if left is empty
void prev() {
  if (fence != head)  // Can't back up from list head
    { fence = fence->prev; leftcnt--; rightcnt++; }
}
};
