// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// Huffman tree node abstract base class
template <typename Elem> class HuffNode {
public:
  virtual ~HuffNode() {}                // Base destructor
  virtual int weight() = 0;             // Return frequency
  virtual bool isLeaf() = 0;            // Determine type
  virtual HuffNode* left() const = 0;   // Get left
  virtual void setLeft(HuffNode*) = 0;  // Set left
  virtual HuffNode* right() const = 0;  // Get right
  virtual void setRight(HuffNode*) = 0; // Set right
};

template <typename Elem>   // Leaf node subclass
class LeafNode : public HuffNode<Elem> {
private:
  KVpair<int,Elem>* it;         // Frequency key-value pair
public:
  LeafNode(int freq, const Elem& val)   // Constructor
    { it = new KVpair<int,Elem>(freq, val); }
  int weight() { return it->key(); }
  KVpair<int,Elem>* val() { return it; }
  bool isLeaf() { return true; }
  virtual HuffNode<Elem>* left() const { return NULL; }
  virtual void setLeft(HuffNode<Elem>*) { }
  virtual HuffNode<Elem>* right() const { return NULL; }
  virtual void setRight(HuffNode<Elem>*) { }
};

template <typename Elem>   // Internal node subclass
class IntlNode : public HuffNode<Elem> {
private:
  HuffNode<Elem>* lc;   // Left child
  HuffNode<Elem>* rc;   // Right child
  int wgt;              // Subtree weight
public:
  IntlNode(HuffNode<Elem>* l, HuffNode<Elem>* r)
    { wgt = l->weight() + r->weight(); lc = l; rc = r; }
  int weight() { return wgt; }
  bool isLeaf() { return false; }
  HuffNode<Elem>* left() const { return lc; }
  void setLeft(HuffNode<Elem>* b)
    { lc = (HuffNode<Elem>*)b; }
  HuffNode<Elem>* right() const { return rc; }
  void setRight(HuffNode<Elem>* b)
    { rc = (HuffNode<Elem>*)b; }
};
